@@ -128,7 +128,6 @@ var DockAbstractAppIcon = GObject.registerClass({
         // and if there are at least 2 monitors.
         if (Docking.DockManager.settings.isolateMonitors &&
             Main.layoutManager.monitors.length > 1) {
-            this._signalsHandler.removeWithLabel('isolate-monitors');
             this._signalsHandler.addWithLabel('isolate-monitors',
                 global.display,
                 'window-entered-monitor',
@@ -284,6 +283,13 @@ var DockAbstractAppIcon = GObject.registerClass({
         this._updateRunningState();
         this._updateFocusState();
         this._updateUrgentWindows(interestingWindows);
+
+        if (Docking.DockManager.settings.isolateWorkspaces) {
+            this._signalsHandler.removeWithLabel('isolate-workspaces');
+            interestingWindows.forEach(window =>
+                this._signalsHandler.addWithLabel('isolate-workspaces',
+                    window, 'workspace-changed', () => this._updateWindows()));
+        }
     }
 
     _updateRunningState() {
@@ -398,15 +404,18 @@ var DockAbstractAppIcon = GObject.registerClass({
                     let monitor_index = Main.layoutManager.findIndexForActor(this);
                     let workArea = Main.layoutManager.getWorkAreaForMonitor(monitor_index);
                     let position = Utils.getPosition();
+                    const { scaleFactor } = St.ThemeContext.get_for_stage(global.stage);
                     this._isHorizontal = ( position == St.Side.TOP ||
                                            position == St.Side.BOTTOM);
                     // If horizontal also remove the height of the dash
                     const { dockFixed: fixedDock } = Docking.DockManager.settings;
-                    let additional_margin = this._isHorizontal && !fixedDock ? Main.overview.dash.height : 0;
+                    const additionalMargin = (this._isHorizontal && !fixedDock) ? Main.overview.dash.height : 0;
                     let verticalMargins = this._menu.actor.margin_top + this._menu.actor.margin_bottom;
+                    const maxMenuHeight = workArea.height - additionalMargin - verticalMargins;
                     // Also set a max width to the menu, so long labels (long windows title) get truncated
-                    this._menu.actor.style = ('max-height: ' + Math.round(workArea.height - additional_margin - verticalMargins) + 'px;' +
-                                              'max-width: 400px');
+                    this._menu.actor.set_style(
+                        `max-width: 400px; ` +
+                        `max-height: ${Math.round(maxMenuHeight / scaleFactor)}px;`);
                 }
             });
             let id = Main.overview.connect('hiding', () => {
@@ -671,10 +680,12 @@ var DockAbstractAppIcon = GObject.registerClass({
             // switching to a different workspace, a launch spinning icon is shown and disappers only
             // after a timeout.
             let windows = this.getWindows();
-            if (windows.length > 0)
+            if (windows.length > 0) {
                 Main.activateWindow(windows[0])
-            else
+            } else {
                 this.app.activate();
+                this.animateLaunch();
+            }
         }
     }
 
@@ -748,8 +759,11 @@ var DockAbstractAppIcon = GObject.registerClass({
         // First activate first window so workspace is switched if needed.
         // We don't do this if isolation is on!
         if (!Docking.DockManager.settings.isolateWorkspaces &&
-            !Docking.DockManager.settings.isolateMonitors)
+            !Docking.DockManager.settings.isolateMonitors) {
+            if (!this.running)
+                this.animateLaunch();
             this.app.activate();
+        }
 
         // then activate all other app windows in the current workspace
         let windows = this.getInterestingWindows();
